use strict;
use warnings;
use Getopt::Long qw(:config posix_default bundling);
use Image::ExifTool;
use Data::Dumper;

my %args;
GetOptions(
	'K|keyword=s@' => \$args{keywords},
	'C|comment=s' => \$args{comment},
	'T|title=s'   => \$args{caption},
	'R|rating=i'  => \$args{rating},
) || die;
my $file = shift || die;
! defined $args{$_} && delete $args{$_} for ( keys %args );
set_image_info( $file,%args );

sub set_image_info {
	my ($file,%args) = @_;
	return if ! %args; 
	delete $args{force};

	my $rating = delete $args{rating};
	my $keywords = delete $args{keywords};
	my $comment = delete $args{comment};
	my $caption = delete $args{caption};

	if ( $rating ) {
		# assign rating to urgncy level same way as digikam 0.9.3
		$rating = int( $rating * -1.4+8.7 );
		$rating = undef if $rating<0;
	}

	my $x = Image::ExifTool->new;
	$x->ExtractInfo($file);
	my $old_rating = $x->GetValue( 'Urgency' );
	my $old_keywords = $x->GetValue( 'Keywords','ValueConv' );
	$old_keywords = [ $old_keywords ] if defined $old_keywords && !ref($old_keywords);
	my $old_comment = $x->GetValue( 'UserComment' );
	my $old_caption = $x->GetValue( 'FileName' );
	my $changed = 0;

	if ( $rating ) {
		if ( ! defined $old_rating or $rating != $old_rating ) {
			$x->SetNewValue( 'Urgency', $rating );
			$changed++;
		}
	} elsif ( defined $old_rating ) {
		$x->SetNewValue( 'Urgency' );
		$changed++;
	}

	if ($keywords && @$keywords) {
		if ( ! defined $old_keywords or ! @$old_keywords or 
			join( '\000', sort @$old_keywords ) ne join( '\000',sort @$keywords)) {
			$x->SetNewValue( 'Keywords', $keywords );
			$changed++;
		}
	} elsif ( defined $old_keywords ) {
		$x->SetNewValue( 'Keywords' );
		$changed++;
	}

	if ( defined $comment && $comment =~m{\S} ) {
		if ( ! defined $old_comment || $comment ne $old_comment ) {
			$x->SetNewValue( 'UserComment', $comment );
			$changed++;
		}
	} elsif ( defined $old_comment ) {
		$x->SetNewValue( 'UserComment' );
		$changed++;
	}

	if ( defined $caption && $caption =~m{\S} ) {
		if ( !defined $old_caption || $caption ne $old_caption ) {
			$x->SetNewValue( 'By-lineTitle', $caption );
			$changed++;
		}
	} elsif ( defined $old_caption ) {
		$x->SetNewValue( 'By-lineTitle' );
		$changed++;
	}

	if ( $changed ) {
		unlink( "$file.bak" );
		rename($file,"$file.bak") or die $!;
		$x->WriteInfo( "$file.bak",$file );
	}
}
